require 'rubygame'
require 'singleton'

class Announce
  include Singleton
  
  def initialize
    @list = []
  end
  
  def draw(surf)
    @list.each { |a| a.draw(surf) }
  end
  
  def announce(string,profit=nil,delay=3)
    @list.push Announcement.new(string,@list.size,profit,delay)
    puts "#{string} #{profit}"
  end
  
  def update
    time = Time.now.to_f
    old_size = @list.size
    @list.reject! { |a| a.update(time) }
    diff = old_size - @list.size
    @list.each { |a| a.y += a.height * diff }
  end
end

class Announcement
  attr_accessor :label, :y, :alpha, :tick, :height
  
  @@font = nil
  
  def initialize(label,index,profit,delay)
    @delay = delay
    @@font ||= Rubygame::TTF.new("res/fonts/FreeSans.ttf",12)
    if profit
      @profit = @@font.render("$#{profit.abs}",true,profit < 0 ? [120,20,20] : [20,120,20])
    end
    @label = @@font.render(label,true,[20,20,20])
    @alpha = 255
    @height = @label.size[1]+10
    @y = Settings.instance[:height] - 4 - (20 + (index * @height))
    @width = @label.size[0]
    @width += 5 + @profit.size[0] if not @profit.nil?
    @tick = Time.now.to_f
    @dead = false
  end
  
  def draw(surf)
    return if dead?
    x = 5
    surf.draw_box_s( [x-4,@y-2],[x + 4 + @width, @y + @label.size[1]+2], [220,220,220,@alpha])
    surf.draw_box( [x-4,@y-2],[x + 4 + @width, @y + @label.size[1]+2], [170,170,170,@alpha])
    @label.set_alpha(@alpha)
    @label.blit(surf,[x,@y])
    if not @profit.nil?
      @profit.set_alpha(@alpha)
      @profit.blit(surf,[x+@label.size[0]+2,@y])
    end
  end
  
  def dead?
    @dead
  end
  
  def kill
    @dead = true
    @alpha = 0
  end
  
  def update(time)
    if @alpha == 255
      if time - @tick > @delay
        @alpha -= 1
      end
    elsif time - @tick > 0.025
      @alpha -= 5
      if @alpha <= 0
        self.kill
      end
    end
    
    return self.dead?
  end
end
