require 'rubygame'
require 'singleton'
require 'lib/visible'
require 'lib/settings'
require 'lib/company'

class BuyMenu
  include Singleton
  attr_reader :buttons
  
  def initialize
    settings = Settings.instance
    @used = 80000
    @new = 110000
    
    @buttons = [BuyButton.new("USED",@used,self,:buy_used), BuyButton.new("NEW",@new,self,:buy_new)]
    
    @buttons.each_with_index do |button,i|
      button.rect.x = settings[:width]/2 - button.rect.w/2
      button.rect.y = settings[:height]/2 - button.rect.h/2 - (i+0.5)*button.rect.h
    end
    
    @company = Company.instance
  end
  
  def show
    @buttons[1].visible = true if @new < @company.cash
    @buttons[0].visible = true if @used < @company.cash
    return false if @new > @company.cash and @used > @company.cash
    return true
  end
  
  def hide
    @buttons.each { |button| button.visible=false }
  end
  
  def buy_used
    @company.cash -= @used
    @company.used_trucks += 1

    @used += (500*rand).to_i + 500
    @buttons[0].redraw(@used)
  end
  
  def buy_new
    @company.new_trucks += 1
    @company.cash -= @new
  end
end

class BuyButton
  include Rubygame::Sprites::Sprite
  include Visible
  
  def initialize(string,cost,callback,method)
    super()
    @visible=false
    @bg = Rubygame::Surface["menu.png"]
    @image = Rubygame::Surface.new(@bg.size)
    @rect = @image.make_rect
    @font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",24)
    
    @string = string
    @callback = callback
    @method = method
    
    self.redraw(cost)
  end
  
  def redraw(cost)
    @image.fill([170,170,170])
    @image.set_colorkey([170,170,170])
    @bg.blit(@image,[0,0])
    text = @font.render("Buy a #{@string} cab for $#{cost}",true,[20,20,20])
    text.blit(@image,[@image.size[0]/2-text.size[0]/2,@image.size[1]/2-text.size[1]/2])
  end
  
  def update
  
  end
  
  def click
    @callback.send(@method)
  end
end
