require 'rubygame'
require 'singleton'
require 'lib/visible'
require 'lib/settings'

class CargoOverlay
  include Rubygame::Sprites::Sprite
  include Singleton
  include Visible
  
  attr_reader :cmd
  
  def initialize
    super
    
    @orig_image = Rubygame::Surface["menu.png"]
    @options = {}
    @font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",24)
    @labels = []
    @settings = Settings.instance
    @info = @font.render("What do you want to do at this city?",true,[10,10,10])
    @cmd = nil
  end
  
  def show(city)
    @options = gen_options(city)
    @info = @font.render("What do you want to do at #{city.name}?",true,[10,10,10])
    self.visible=true
    @alpha = Array.new(@options.keys.size,128)
    @boxes = []
    @height = 50

    @labels = @options.keys.sort.map do |label|
      @font.render(label,true,[20,20,20])
    end
    @x = @settings[:width]/2 - @labels[0].size[0]/2 - 50
    @y = @settings[:height]/2 - @height*(@labels.size/2.0)
    
    @labels.each_with_index { |label,index|
      tl = [@x-5,@y+@height*index-2]
      br = [@x+label.size[0]+5,@y+@height*index+label.size[1]+2]
      rect = Rect.new(tl[0],tl[1],br[0]-tl[0],br[1]-tl[1])
      @boxes.push [rect,@options.keys.sort[index]]
    }
  end
  
  def gen_options(city)
    options = {"Sell cargo" => nil }
    city.produces.each { |item| options["Sell cargo and buy #{item}"] = item }
    return options
  end
  
  def hide
    self.visible=false
    self.draw(@screen) if @screen
  end
  
  def mouse_at(x,y)
    @boxes.each_with_index do |pair,index|
      box,cmd = *pair
      if box.collide_point?(x,y)
        if @alpha[index] != 255
          @alpha = Array.new(@options.keys.size,128)
          @alpha[index] = 196
        end
        return
      end
    end
  end
  
  def click_at(x,y)
    @boxes.each_with_index do |pair,index|
      box,cmd = *pair
      if box.collide_point?(x,y)
        @cmd = @options[cmd]
        return true
      end
    end
    return false
  end
  
  def draw(surf)
    @screen ||= surf
    if @visible
      @info.blit(surf,[@settings[:width]/2-@info.size[0]/2,5])
      @labels.each_with_index do |label,index|
        tl = [@x-5,@y+@height*index-2]
        br = [@x+label.size[0]+5,@y+@height*index+label.size[1]+2]
        surf.draw_box_s(tl,br,[255,255,255,@alpha[index]])
        surf.draw_box(tl,br,[60,60,60,@alpha[index]])
        label.blit(surf,[@x,@y+@height*index])
      end
    end
  end
  
  def update
  
  end
end
