require 'singleton'
require 'rubygame'
require 'lib/announce'
require 'lib/visible'

class Company
  include Singleton
  attr_reader :cash, :used_trucks, :new_trucks, :trucks, :overlay, :group
  
  def initialize
    @trucks = Rubygame::Sprites::Group.new
    @overlay = [CashOverlay.instance,TruckOverlay.instance]
    self.cash = 200000
    @new_trucks = 0
    @used_trucks = 0
    self.used_trucks = 0
    self.new_trucks = 0
    @group = nil
    
    @last_called = nil
  end
  
  def group=(group)
    @group=group
  end
  
  def add_truck(route,orders)
    truck = Truck.new(route,orders)
    truck.show
    Day.add_task(Day.day+3,Proc.new {truck.maintenance(@last_called)})
    @group.push truck
    @trucks.push truck
  end
  
  def cash=(n)
    @cash = n
    @overlay[0].cash = n
  end
  
  def new_trucks=(n)
    @new_trucks = n
    @overlay[1].trucks = @new_trucks + @used_trucks
    
    @last_called = :new
  end
  
  def used_trucks=(n)
    @used_trucks = n
    @overlay[1].trucks = @new_trucks + @used_trucks

    @last_called = :old
  end
end

class Truck
  include Rubygame::Sprites::Sprite
  include Visible
  
  def maintenance(type)
    cost = (@miles-500+(rand*1000).to_i) * (type == :new ? 0.1 : 0.2)
    Company.instance.cash -= cost
    Announce.instance.announce("Truck maintenance costs ",((-cost*100).to_i).to_f/100)
    Day.add_task(Day.day+3,Proc.new{self.maintenance(type)})
    @total_miles += @miles
    @miles = 0
    @maintenance_total += -cost
  end
  
  @@orig_image = nil
  @@sound = nil
  
  attr_reader :image, :rect, :route, :orders, :arrivals, :maintenance_total
  
  def total_miles
    @total_miles + @miles
  end
  
  def initialize(route,orders)
    super()
    @total_miles = 0
    @maintenance_total = 0
    @arrivals = 0
    @route = route
    @orders = orders
    @@sound ||= Rubygame::Sound.autoload('money.ogg')
    @@orig_image ||= Rubygame::Surface['truck.png']
    
    @gas = 600
    @prev = 0
    @dest = 1
    @cargo = nil
    @last_update = Time.now.to_f
    @miles = 0
    
    @highways = Map.instance.highways
    
    start(@prev,@dest)
  end
  
  def start(a,b)
    profit = 0
    if @orders.keys.include?(@route[a])
      amt = 20 + 15 * rand
      if not @cargo.nil?
        m_amt = amt
        m_amt /= 2 if @route[a].produces.include?(@cargo)
        m_amt *= 3 if @route[a].consumes.include?(@cargo)
        profit = @route[a].prices[@cargo] * m_amt
        @arrivals += 1
      end
      if not @orders[@route[a]].nil?
        @cargo = @orders[@route[a]]
        cost = amt * @route[a].prices[@cargo]
        profit -= cost
      end
    end
    gas_cost = @orig_distance.nil? ? 0 : (4 * @orig_distance) * 2.00 + (rand * 1.5) # Gas
    profit -= gas_cost
    Company.instance.cash += profit
    Announce.instance.announce("Truck arrives in #{@route[a].name} for ",((profit*100).to_i).to_f/100)

    @distance = ((@route[a].real_x - @route[b].real_x)**2 + (@route[a].real_y - @route[b].real_y)**2)**0.5
    @miles += @distance
    @x_rate = (@route[b].real_x - @route[a].real_x)/@distance
    @y_rate = -(@route[b].real_y - @route[a].real_y)/@distance
    @orig_distance = @distance

    if @x_rate > 0
      if @y_rate > 0
        @degree = -90 + ((360/(Math::PI * 2)) * ( Math.asin(@y_rate)))
      else
        @degree = -90 - ((360/(Math::PI * 2)) * ( Math.asin(-@y_rate)))
      end
    else
      if @y_rate > 0
        @degree = 90 - ((360/(Math::PI * 2)) * ( Math.asin(@y_rate)))
      else
        @degree = 90 + ((360/(Math::PI * 2)) * ( Math.asin(-@y_rate)))
      end
    end

    @image = @@orig_image.rotozoom(@degree,1,true)
    @pos = [@route[a].real_x-@image.size[0]/2,@route[a].real_y-@image.size[1]/2]
    @rect = @image.make_rect
    @rect.x,@rect.y = *@pos
  end
  
  def update
    return if Settings.instance[:paused]
    tick = Time.now.to_f
    if tick - @last_update > 0.125
      if @distance > 0
        mod = (@highways.include?([@route[@prev],@route[@dest]]) ? 2 : 1)
        @rect.x += @x_rate * mod
        @rect.y -= @y_rate * mod
        @distance -= mod
      else # Arrived!
        @prev = @dest
        @dest = (@dest >= @route.size-1 ? 0 : @dest+1)
        start(@prev,@dest)
        @@sound.play
      end
      @last_update = tick
    end
  end  
end

class TruckOverlay
  include Singleton
  include Rubygame::Sprites::Sprite
  
  def initialize
    super
    @font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",36)

    @orig_image = Rubygame::Surface["interstate_small.png"]
    @image = Rubygame::Surface.new(@orig_image.size)
    @image.set_colorkey([170,170,170])
    @rect = @image.make_rect
  end
  
  def trucks=(n)
    @image.fill([170,170,170])
    @orig_image.blit(@image,[0,0])
    @count = @font.render("#{n}",true,[255,255,255])
    @count.blit(@image,[@rect.w/2-@count.size[0]/2,8])
  end
  
  def update
  
  end
end

class CashOverlay
  include Singleton
  include Rubygame::Sprites::Sprite
  
  def initialize
    super
    
    @font = Rubygame::TTF.new("res/fonts/astronaut.ttf",36)
    @cash = 0
  end
  
  def cash=(n)
    @goal = n
    redraw
  end
  
  def redraw
    color = @cash > 0 ? [20,80,20] : [80,20,20]
    @image = @font.render("$#{@cash.to_i}",true,color)
    @rect = @image.make_rect
  end

  def update
    if @cash > @goal
      @cash -= 10173
      @cash = @goal if @cash < @goal
      redraw
    elsif @cash < @goal
      @cash += 10291
      @cash = @goal if @cash > @goal
      redraw
    end
  end
end
