require 'rubygems'
require 'rubygame'

require 'lib/map'
require 'lib/popup'
require 'lib/company'
require 'lib/buymenu'
require 'lib/routeoverlay'
require 'lib/cargooverlay'
require 'lib/splashoverlay'
require 'lib/helpoverlay'
require 'lib/announce'
require 'lib/showrouteoverlay'
require 'lib/quitoverlay'
require 'lib/retireoverlay'

require 'lib/controller'
require 'lib/controller/splashcontroller'

require 'lib/settings'
require 'lib/day'
require 'lib/disasters'

include Rubygame

MusicCount = 4

class Core
  attr_reader :sprites, :screen

  def initialize
    srand(Time.now.to_i)
    Rubygame.init
    
    settings = Settings.instance
    settings[:width] = 1024
    settings[:height] = 768
    
    @screen = Rubygame::Screen.set_mode([settings[:width],settings[:height]], 0, [Rubygame::HWSURFACE,Rubygame::DOUBLEBUF])
    @screen.title = "Interstate Trucking"
    @screen.show_cursor = true
    
    Surface.autoload_dirs = [ 'res/art', 'maps/I20/art' ]
    
    TTF.setup
    @queue = EventQueue.new
    @clock = Clock.new { |clock| clock.target_framerate = 30 }
    
    @sprites = Sprites::Group.new
    
    @background = Surface.new(@screen.size)
    @background.fill([235,233,225])
    @map = Map.instance
    
    @company = Company.instance
    @company.group = @sprites
    @company.overlay[1].rect.x = @screen.width - @company.overlay[1].rect.width - 5
    @company.overlay[1].rect.y = 5
    
    @sprites.push(@map)
    @sprites.push *@map.cities
    @sprites.push *@company.overlay
    @sprites.push *BuyMenu.instance.buttons
    @sprites.push RouteOverlay.instance
    @sprites.push ShowRouteOverlay.instance
    @sprites.push CargoOverlay.instance
    @sprites.push SplashOverlay.instance
    @sprites.push HelpOverlay.instance
    @sprites.push QuitOverlay.instance
    @sprites.push RetireOverlay.instance
    @sprites.push Popup.instance
    
    Controller.callback=(self)
    Controller.add_context("map", @map)
    @controller = SplashController
    
    @announce = Announce.instance
    
    Rubygame::Music.autoload_dirs=["res/music"]
    Rubygame::Sound.autoload_dirs=["res/music"]
    @music = Rubygame::Music.autoload("background#{1+(MusicCount*rand).to_i}.ogg")
    @music.play
    
    @time = Time.now.to_i
    Day.add_task(Day.day+1,Proc.new { Disaster.occur })
  end
  
  def controller=(control)
    @controller = control
  end
  
  def main_loop
    catch(:quit) do
    loop do
      @clock.tick
      @queue.each do |event|
        case event
          when QuitEvent
            return
        end
        
        @controller.handle(event)
      end
#      puts "\e[H\e[2J#{@clock.framerate}"
      @background.blit(@screen, [0,0])
      
      @sprites.update
      @sprites.draw(@screen)
      
      @announce.update
      @announce.draw(@screen)
      screen.flip
      
      update_music
      tick = Time.now.to_i
      if tick - @time > 60
        @time = tick
        Day.next_day
      end
    end
    end
  end
  
  def update_music
    if @music.respond_to?(:playing?)
      return if @music.playing?
      
      @music = Time.now.to_i + (30 + 60*rand).to_i
      return
    end
    
    if Time.now.to_i > @music
      begin
        @music = Rubygame::Music.autoload("background#{1+(MusicCount*rand).to_i}.ogg")
        @music.play if not @music.nil?
      rescue
        ;
      end
    end
  end
  
  def quit
    Rubygame.quit
  end
end
