require 'rubygame'
require 'singleton'
require 'lib/visible'
require 'lib/settings'
require 'yaml'

class RetireOverlay
  include Singleton
  include Rubygame::Sprites::Sprite
  include Visible

  def initialize
    super
    @settings = Settings.instance

    @visible=false
    begin
      @best = YAML.load_file(File.join(ENV["HOME"],".interstate_trucking.yaml"))
    rescue
      @best = {}
    end
  end
  
  def show
    super
    font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",24)
    Company.instance.trucks.map { |t| t.hide }
    @settings[:paused] = true
    nstats = {:trucks => Company.instance.trucks.size,
              :miles => Company.instance.trucks.inject(0) { |sum,t| sum + t.total_miles.to_i },
              :deliveries => Company.instance.trucks.inject(0) { |sum,t| sum + t.arrivals },
              :maintenance => Company.instance.trucks.inject(0) { |sum,t| sum + t.maintenance_total },
              :days => Day.day,
              :balance => (Company.instance.cash*100).to_i/100.0,
              :weddings => (Disaster.state(:brother_marriage) || 0)
             }
    nstats.each_key { |key| @best[key] = nstats[key] if @best[key].nil? or nstats[key] > @best[key] }
    begin
      File.open(File.join(ENV["HOME"],".interstate_trucking.yaml"),"w").puts @best.to_yaml
    rescue
      STDERR.puts "Can't write best.yaml"
      ; # do nothing
    end
    @stats = [
      "Trucks owned: #{nstats[:trucks]} (Best: #{@best[:trucks] || 0})",
      "Miles traveled: #{nstats[:miles]} (Best: #{@best[:miles] || 0})",
      "Deliveries made: #{nstats[:deliveries]} (Best: #{@best[:deliveries] || 0})",
      "Total maintenance: #{nstats[:maintenance]} (Best: #{@best[:maintenance] || 0})",
      "Days passed: #{nstats[:days]} (Best: #{@best[:days] || 0})",
      "Current balance: $#{nstats[:balance]} (Best: #{@best[:balance] || 0})",
      "Weddings attended: #{nstats[:weddings]} (Worst: #{@best[:weddings] || 0})"
    ].map { |x| font.render(x,true,[20,20,20]) }
  end
  
  def update
  
  end
  
  def draw(surf)
    if visible?
      surf.draw_box_s([10,10],[Settings.instance[:width]-10,Settings.instance[:height]-10],[255,255,255,196])
      @stats.each_with_index { |label,i|
        label.blit(surf,[60,80+label.size[1]*i])
      }
    end
  end
end
