require 'rubygame'
require 'singleton'
require 'lib/visible'
require 'lib/settings'

class RouteOverlay
  include Singleton
  include Rubygame::Sprites::Sprite
  include Visible
  
  attr_reader :stops,:cargo
  
  Messages = ["Click on the first city for this route",
              "Click on the next city in this route (ENTER to save route)"]
  
  def initialize
    super
    self.reset
    @visible = false
    @font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",36)
    @settings = Settings.instance

    @mx,@my = 0
    @screen = nil
    
    self.reset
    self.redraw
  end
  
  def mouse_at(x,y)
    @mx,@my = x,y
  end
    
  def redraw
    @image = @font.render(Messages[@index],true,[10,10,10])
    @rect = @image.make_rect
    @rect.x = @settings[:width]/2 - @rect.w/2
  end
  
  def add_stop(city)
    @stops.push(city)
    @index = 1
    self.redraw
  end
  
  def reset
    @index = 0
    @stops = []
    @cargo = []
  end
  
  def draw(surf)
    if visible?
      @screen ||= surf
      line_color = [64,196,32,32]
  
      r = super(surf)
      @stops.each_with_index do |stop,i|
        if @stops[i+1]
          [-1,0,1].each { |t| surf.draw_line_a([stop.rect.x+t+stop.rect.width/2,stop.rect.y+6],[@stops[i+1].rect.x+t+@stops[i+1].rect.w/2,@stops[i+1].rect.y+6],line_color) }
        end
      end
      if @stops.size > 0
      [-1,0,1].each { |t| surf.draw_line_a([@stops[@stops.size-1].rect.x+t+@stops[@stops.size-1].rect.width/2,@stops[@stops.size-1].rect.y+6],[@mx+t,@my],line_color) }
      end
      return r
    end
  end
  
  def update
  
  end
  
  def show
    self.visible=true
  end
  
  def hide
    self.visible=false
    self.draw(@screen)
  end
end
