require 'rubygame'
require 'singleton'
require 'lib/visible'
require 'lib/settings'

class ShowRouteOverlay
  include Singleton
  include Rubygame::Sprites::Sprite
  include Visible
  
  def initialize
    super

    @visible = false
    font = Rubygame::TTF.new("res/fonts/FreeSans.ttf",24)
    @label = font.render("Change route",true,[20,20,20])

    settings = Settings.instance
    @x = settings[:width]-@label.size[0]-12
    @y = settings[:height]-@label.size[1]-12
    @rect = Rubygame::Rect.new(@x-5,@y-5,@label.size[0]+10,@label.size[1]+10)
    @alpha = 128

    @screen = nil
    @truck = nil
  end
  
  def show(truck)
    super()
    @truck = truck
  end
  
  def mouse_at(x,y)
    if @rect.collide_point?(x,y)
      @alpha = 255
    else
      @alpha = 128
    end
  end

  def click_at(x,y)
    if @rect.collide_point?(x,y)
      Company.instance.trucks.delete(@truck)
      Company.instance.group.delete(@truck)
      @truck = nil
      return true
    end
    return false
  end
  
  def draw(surf)
    if visible? and not @truck.nil?
      @screen ||= surf
      line_color = [0,0,255,96]
  
      @truck.route.each_with_index do |stop,i|
        if @truck.route[i+1]
          [-1,0,1].each { |t| surf.draw_line_a([stop.rect.x+t+stop.rect.width/2,stop.rect.y+6],[@truck.route[i+1].rect.x+t+@truck.route[i+1].rect.w/2,@truck.route[i+1].rect.y+6],line_color) }
        else
          [-1,0,1].each { |t| surf.draw_line_a([stop.rect.x+t+stop.rect.width/2,stop.rect.y+6],[@truck.route[0].rect.x+t+@truck.route[0].rect.w/2,@truck.route[0].rect.y+6],line_color) }
        end
      end
      
      surf.draw_box_s([@x-5,@y-5],[@x+@label.size[0]+5,@y+@label.size[1]+5],[255,255,255,@alpha])
      surf.draw_box([@x-5,@y-5],[@x+@label.size[0]+5,@y+@label.size[1]+5],[160,160,160,@alpha])
      @label.blit(surf,[@x,@y])
    end
  end
  
  def update
  
  end
  
  def hide
    self.visible=false
    self.draw(@screen)
  end
end
